@@ -29,21 +29,15 @@
   (define (with-mock-wrapper func)
     (lambda (obj)
       (cond ((mock? obj)
-	     (dynamic-wind
-		 coverlets
-		 (lambda ()
-		   (func (obj 'value)))
-		 openlets))
+	     (let-temporarily (((*s7* 'openlets) #f))
+	       (func (obj 'value))))
 
 	    ((not (openlet? obj))
 	     (func obj))
 
 	    ((procedure? obj) ; TODO: and c-pointer? c-object?
-	     (dynamic-wind
-		 coverlets
-		 (lambda ()
-		   (func obj))
-		 openlets))
+	     (let-temporarily (((*s7* 'openlets) #f))
+	       (func obj)))
 
 	    (else
 	     (let ((func-name (string->symbol (object->string func))))
@@ -53,24 +47,29 @@
 
   (define (with-mock-wrapper* func)
     (lambda args
-      (let* ((unknown-openlets #f)
-	     (new-args (map (lambda (arg)
-			      (if (mock? arg)
-				  (arg 'value)
-				  (begin
-				    (if (and (openlet? arg)
-					     (not (procedure? arg))
-					     (not (c-pointer? arg)))
-					(set! unknown-openlets #t))
-				    arg)))
-			    args)))
+      (let ((unknown-openlets #f)
+	    (new-args ()))
+	(for-each (lambda (arg) ; not map here because (values) should not be ignored: (+ (mock-number 4/3) (values))
+		    (set! new-args
+			  (cons (if (mock? arg)
+				    (arg 'value)
+				    (begin
+				      (if (and (openlet? arg)
+					       (not (procedure? arg))
+					       (not (macro? arg))
+					       (not (c-pointer? arg)))
+					  (set! unknown-openlets #t))
+				      arg))
+				new-args)))
+		  args)
 	(if unknown-openlets
-	    (apply func new-args)
-	    (dynamic-wind coverlets (lambda () (apply func new-args)) openlets)))))
+	    (apply func (reverse! new-args))
+	    (let-temporarily (((*s7* 'openlets) #f)) 
+	      (apply func (reverse! new-args)))))))
 
   ;; one tricky thing here is that a mock object can be the let of with-let: (with-let (mock-port ...) ...)
-  ;;   so a mock object's method can be called even when no argument is a mock object.  Even trickier, the
-  ;;   mock object can be a closure's containing (open)let: (display (openlet (with-let (mock-c-pointer 0) (lambda () 1))))
+  ;;   so a mock object's method can be called even when no argument is a mock object.  Even trickier
+  ;;   (display (openlet (with-let (mock-c-pointer 0) (lambda () 1))))
   ;; --------------------------------------------------------------------------------
 
   (set! *mock-vector*
@@ -113,7 +112,9 @@
 		       'format             (with-mock-wrapper* #_format)
 		       'write              (with-mock-wrapper* #_write)
 		       'display            (with-mock-wrapper* #_display)
+		       'vector-dimension   (with-mock-wrapper #_vector-dimension)
 		       'vector-dimensions  (with-mock-wrapper #_vector-dimensions)
+		       'vector-rank        (with-mock-wrapper #_vector-rank)
 		       'fill!              (with-mock-wrapper* #_fill!)
 		       'vector-fill!       (with-mock-wrapper* #_vector-fill!)
 		       'vector->list       (with-mock-wrapper* #_vector->list)
@@ -286,10 +287,10 @@
 						     (if (string? str)
 							 (length str)
 							 (error 'wrong-type-arg "string-length argument should be a string: ~A" str)))
-						   (with-mock-wrapper #_string-length))
+						   (with-mock-wrapper #_length))
 
 		       'string-append          (with-mock-wrapper* #_string-append)
-		       'string-copy            (with-mock-wrapper #_copy)
+		       'string-copy            (with-mock-wrapper #_copy) ; new form -> with-mock-wrapper* ? 
 		       
 		       'string=?               (with-mock-wrapper* #_string=?)
 		       'string<?               (with-mock-wrapper* #_string<?)
@@ -336,7 +337,7 @@
 		       'string-ci<=?           (with-mock-wrapper* #_string-ci<=?)
 		       'string-ci>=?           (with-mock-wrapper* #_string-ci>=?)
 		       'string?                (with-mock-wrapper #_string?)
-		       'length                 (with-mock-wrapper #_string-length)
+		       'length                 (with-mock-wrapper (if (provided? 'pure-s7) #_length #_string-length))
 		       'append                 (with-mock-wrapper* #_append)
 		       'class-name             '*mock-string*)))
 	  
@@ -411,6 +412,7 @@
 		       'char-ci>=?         (with-mock-wrapper* #_char-ci>=?)
 		       'string             (with-mock-wrapper* #_string)
 		       'string-fill!       (with-mock-wrapper* #_string-fill!)
+		       'fill!              (with-mock-wrapper* #_fill!)
 		       'object->string     (with-mock-wrapper* #_object->string)
 		       'format             (with-mock-wrapper* #_format)
 		       'write              (with-mock-wrapper* #_write)
@@ -474,7 +476,7 @@
 		 'negative?        (with-mock-wrapper #_negative?)
 		 'infinite?        (with-mock-wrapper #_infinite?)
 		 'nan?             (with-mock-wrapper #_nan?)
-		 'append           (with-mock-wrapper* #_append)
+		 ;'append           (with-mock-wrapper* #_append) ;?? (append ... 3 ...) is an error
 		 'magnitude        (with-mock-wrapper #_magnitude)
 		 'angle            (with-mock-wrapper #_angle)
 		 'rationalize      (with-mock-wrapper* #_rationalize)
@@ -530,9 +532,10 @@
 		 '<=               (with-mock-wrapper* #_<=)
 		 '>=               (with-mock-wrapper* #_>=)
 		 
-		 'make-polar       (if (provided? 'pure-s7)
-				       (lambda (mag ang) (#_complex (* mag (cos ang)) (* mag (sin ang))))
-				       (lambda args (apply #_make-polar (map ->value args))))
+ 		 'make-polar       (if (provided? 'pure-s7)
+ 				       (lambda (mag ang) (#_complex (* mag (cos ang)) (* mag (sin ang))))
+ 				       (lambda (mag ang) (#_make-polar (->value mag) (->value arg))))
+
 		 'make-rectangular (with-mock-wrapper* #_complex)
 		 'complex          (with-mock-wrapper* #_complex)
 		 'random-state     (with-mock-wrapper* #_random-state)
@@ -559,6 +562,7 @@
 		 'string->list     (with-mock-wrapper* #_string->list)
 		 'substring        (with-mock-wrapper* #_substring)
 		 'vector-fill!     (with-mock-wrapper* #_vector-fill!)
+		 'fill!            (with-mock-wrapper* #_fill!)
 		 'make-string      (with-mock-wrapper* #_make-string)
 		 'string-ref       (with-mock-wrapper* #_string-ref)
 		 'string-set!      (with-mock-wrapper* #_string-set!)
@@ -618,7 +622,6 @@
   ;; from Wikipedia:
   ;; x + y =	[a+c, b+d]
   ;; x - y =	[a-d, b-c]
-  ;; x  y =	[min(ac, ad, bc, bd), max(ac, ad, bc, bd)]
   ;; x / y =	[min(a/c, a/d, b/c, b/d), max(a/c, a/d, b/c, b/d)]
   
   (define *interval*
@@ -866,7 +869,8 @@
 	       (mock-symbol-class
 		(inlet 'equivalent?           (with-mock-wrapper* #_equivalent?)
 		       'gensym?               (with-mock-wrapper #_gensym?)
-		       'append                (with-mock-wrapper* #_append)
+		       ;'append               (with-mock-wrapper* #_append) ;? (append ... 'a ...) is an error
+		       'fill!                 (with-mock-wrapper* #_fill!)
 		       'symbol->string        (with-mock-wrapper #_symbol->string)
 		       'symbol->value         (with-mock-wrapper* #_symbol->value)
 		       'symbol->dynamic-value (with-mock-wrapper #_symbol->dynamic-value)
@@ -917,6 +921,7 @@
 		       'format          (with-mock-wrapper* #_format)
 		       'write           (with-mock-wrapper* #_write)
 		       'display         (with-mock-wrapper* #_display)
+		       'fill!           (with-mock-wrapper* #_fill!)
 		       )))
 	  
 	  (define* (mock-c-pointer (int 0) type info weak1 weak2)
@@ -1005,7 +1010,7 @@
 		       'output-port?        (with-mock-wrapper #_output-port?)
 		       'port-closed?        (with-mock-wrapper #_port-closed?)
 		       'equivalent?         (with-mock-wrapper* #_equivalent?)
-		       'append              (with-mock-wrapper* #_append)
+		       ;'append              (with-mock-wrapper* #_append) ; ?? (append (open-input-string "asdf")...) is an error
 		       'set-current-output-port (with-mock-wrapper #_set-current-output-port)
 		       'set-current-input-port  (with-mock-wrapper #_set-current-input-port)
 		       'set-current-error-port  (with-mock-wrapper #_set-current-error-port)
@@ -1098,4 +1103,4 @@
   ((p 'owner) p) -> "bil"
 |#
 
-  ) ; end of outer let
+  #f) ; end of outer let
